% Load image and scale down a bit
im = imresize(im2double(imread('max-schmeling-landscape-cropped.png')),0.5);
if exist('max-scribble.mat','file')
  load('max-scribble.mat','S','SI');
else
  [SI,S] = get_scribbles(im);
end
assert(size(SI,1) == size(im,1) && size(SI,2) == size(im,2));

% build an original image with scribble overlay
orig_w_scrib = bsxfun(@plus,bsxfun(@times,SI==0,im), ind2rgb(SI+1,[0 0 0;C]));

% number of subplots
nsp = 5;
subplot(1,nsp,1);
imshow(orig_w_scrib);
title('original with scribbles');
set(gca,'Position',[1-(nsp-1+1)/(nsp) 0 0.9/(nsp) 1])
drawnow;

% Some color values
C = rand(max(SI(SI~=0)),3);
if size(C,1) == 2
  C = [0 0 0;0 1 0];
else
  % else <6 values
  C(1:6,:) = [0 0 0.5;0.5 0 0;0.3 0.25 0.2;0.75 0.75 0.5;0.5 0 0;0 0.75 0.75];
  C = C(1:max(SI(SI~=0)),:);
end

% Create a 2d mesh
[V,F] = create_regular_grid(size(im,2),size(im,1),0,0); 
V(:,1) = V(:,1)*size(im,2);
V(:,2) = (1-V(:,2))*size(im,1);
% append extra color-data coordinates
extra = reshape(1*max(V(:))*im(:),size(im));
V =[V reshape(extra,[],size(im,3))];

subplot(1,nsp,2);
% plot mesh with image color as height (and color)
t = trisurf(F,V(:,1),V(:,2),V(:,3),'EdgeColor','none','FaceColor','interp');
% turn on lights
l = light('Position',[-1.0,-1.0,100.0],'Style','infinite');
lighting phong;
% set camera view
view(29,34);
% set perspective view
camproj('perspective');
hold on;
% also plot base mesh
b = trisurf(F,V(:,1),V(:,2),0*V(:,3),V(:,3),'EdgeColor','none','FaceColor','interp');
hold off;
% Set grayscale colormap
axis vis3d;
colormap(gray(255));
title('image surface');
set(gca,'Position',[1-(nsp-2+1)/(nsp) 0 0.9/(nsp) 1])
drawnow;

% Set up colorization boundary conditions for each scribble
b = find(SI~=0);
bc = sparse(1:numel(SI(SI~=0)),SI(SI~=0),1,numel(SI(SI~=0)),max(SI(SI~=0)));

% Compute weights for each scribble
weight_method = 'bbw';
% Bounded biharmonic weights are probably overkill for colorization: they cost
% more to compute than harmonic weights. Perhaps not even correct in terms of
% motivating their properties.
W = zeros(size(V,1),max(SI(:)));
fprintf('Using %s weights\n',weight_method);
switch weight_method
case 'harmonic'
  W = harmonic(V,F,b,bc,1);
case 'bbw'
  W = biharmonic_bounded(V,F,b,bc);
end

subplot(1,nsp,3);
% Scibble of interest
ii = min(size(W,2),4);
% build a weight image with scribble overlay
Wii_w_scrib = ...
  bsxfun(@plus,bsxfun(@times,SI==0,reshape(W(:,ii),size(im))), + ...
  ind2rgb((SI==ii).*SI+1,[0 0 0;C]));
imshow(Wii_w_scrib);
title(sprintf('%s weights of scribble #%d',weight_method,ii));
set(gca,'Position',[1-(nsp-3+1)/(nsp) 0 0.9/(nsp) 1])
drawnow;

% Compute colorized image according to weights
CNTSC = rgb2ntsc(C);
outNTSC = rgb2ntsc(repmat(im,[1 1 3/size(im,3)]));
outNTSC(:,:,2) = reshape(W*CNTSC(:,2),size(outNTSC,1),size(outNTSC,2));
outNTSC(:,:,3) = reshape(W*CNTSC(:,3),size(outNTSC,1),size(outNTSC,2));
% Compute final colorized output image
out = ntsc2rgb(outNTSC);

subplot(1,nsp,4);
imshow(out);
title(sprintf('Colorized using %s weights',weight_method));
set(gca,'Position',[1-(nsp-4+1)/(nsp) 0 0.9/(nsp) 1])
drawnow;

% If levin's code is available, then compute their weights to compare against
if exist('getColorExact','file')
  fprintf('Computing colorization using [Levin et al.]...\n');
  % Compute colorized image using levin's code (~special harmonic weights)
  SNTSC = rgb2ntsc(ind2rgb(SI+1,[0 0 0;C]));
  ntscIm = rgb2ntsc(repmat(im,[1 1 3/size(im,3)]));
  ntscIm(:,:,2) = SNTSC(:,:,2);
  ntscIm(:,:,3) = SNTSC(:,:,3);
  % Levin's matlab code could be much faster, but I use it without modification
  % to be sure
  [levin,levinNTSC] = getColorExact(double(SI>0),ntscIm);

  subplot(1,nsp,5);
  imshow(levin);
  title('[Levin et al.]');

else
  fprintf('Download Levin''s code and addpath accordingly\n');

  subplot(1,nsp,5);
  title(sprintf('Download Levin''s code\n and addpath accordingly'));
end
set(gca,'Position',[1-(nsp-5+1)/(nsp) 0 0.9/(nsp) 1])
drawnow;
